# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging
from oslo_utils import versionutils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import constants as const

from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseRecordsetsTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                     'type']

    @classmethod
    def setup_clients(cls):
        super(BaseRecordsetsTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseRecordsetsTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseRecordsetsTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

        # All the recordset tests need a zone, create one to share
        zone_name = dns_data_utils.rand_zone_name(name="TestZone",
                                              suffix=cls.tld_name)
        LOG.info('Create a zone: %s', zone_name)
        cls.zone = cls.zones_client.create_zone(name=zone_name)[1]

    @classmethod
    def resource_cleanup(cls):
        cls.zones_client.delete_zone(
            cls.zone['id'], ignore_errors=lib_exc.NotFound)
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseRecordsetsTest, cls).resource_cleanup()


class RecordsetsTest(BaseRecordsetsTest):

    credentials = ["admin", "primary", "alt",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(RecordsetsTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(RecordsetsTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.admin_zone_client = cls.os_admin.dns_v2.ZonesClient()
        cls.client = cls.os_primary.dns_v2.RecordsetClient()
        cls.alt_client = cls.os_alt.dns_v2.RecordsetClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('631d74fd-6909-4684-a61b-5c4d2f92c3e7')
    def test_create_recordset(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'create_recordset', expected_allowed, True,
            self.zone['id'], recordset_data)

        LOG.info('Create a Recordset')
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])

        LOG.info('Ensure we respond with PENDING')
        self.assertEqual(const.PENDING, body['status'])

    # We cannot use DDT here as these tests are part of the refstack
    # interoperability test suite and need to be unique for traceability.
    def _test_create_recordset_type(self, name, type, records):
        if name is not None:
            recordset_name = name + "." + self.zone['name']

        else:
            recordset_name = self.zone['name']

        recordset_data = {
            'name': recordset_name,
            'type': type,
            'records': records,
        }

        LOG.info('Create a Recordset')
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])

        LOG.info('Ensure we respond with PENDING')
        self.assertEqual(const.PENDING, body['status'])

    @decorators.idempotent_id('d03b69a5-5052-43bc-a38a-b511b6b34304')
    def test_create_recordset_type_A(self):
        self._test_create_recordset_type(
            "www", "A", ["192.0.2.1", "192.0.2.2", "192.0.2.3"])

    @decorators.idempotent_id('ac110198-d58a-4a18-aceb-414d7e513d49')
    def test_create_recordset_type_AAAA(self):
        self._test_create_recordset_type(
            "www", "AAAA", ["2001:db8::1", "2001:db8::1", "2001:db8::"])

    @decorators.idempotent_id('6c22a3f9-3f4d-4b32-bdf2-5237851ed25e')
    def test_create_recordset_type_SRV_TCP(self):
        self._test_create_recordset_type(
            "_sip._tcp", "SRV", [
                "10 60 5060 server1.example{}".format(self.tld_name),
                "20 60 5060 server2.example{}".format(self.tld_name),
                "20 30 5060 server3.example{}".format(self.tld_name)])

    @decorators.idempotent_id('59c1aa42-278e-4f7b-a6a1-4320d5daf1fd')
    def test_create_recordset_type_SRV_UDP(self):
        self._test_create_recordset_type(
            "_sip._udp", "SRV", [
                "10 60 5060 server1.example{}".format(self.tld_name),
                "10 60 5060 server2.example{}".format(self.tld_name),
                "20 30 5060 server3.example{}".format(self.tld_name)])

    @decorators.idempotent_id('1ac46f94-f03a-4f85-b84f-826a2660b927')
    def test_create_recordset_type_CNAME(self):
        self._test_create_recordset_type(
            "alias-of-target", "CNAME", ["target.example.org."])

    @decorators.idempotent_id('bf872487-7975-4a96-bb03-d24e393a0ce8')
    def test_create_recordset_type_MX_at_APEX(self):
        self._test_create_recordset_type(
            None, "MX", ["10 mail1.example.org.", "20 mail2.example.org."])

    @decorators.idempotent_id('96fe72a4-a81c-4a01-a81f-39ebafad115c')
    def test_create_recordset_type_MX_under_APEX(self):
        self._test_create_recordset_type(
            "under", "MX", ["10 mail.example.org."])

    @decorators.idempotent_id('481496f1-917a-40d5-89fd-4a3794c24215')
    def test_create_recordset_type_SSHFP(self):
        self._test_create_recordset_type(
            "www", "SSHFP", ["2 1 123456789abcdef67890123456789abcdef67890"])

    @decorators.idempotent_id('8e7ecedb-5c35-46f8-ae0e-39e4aaabc97d')
    def test_create_recordset_type_TXT(self):
        self._test_create_recordset_type(
            "sample", "TXT", ["\"Any Old Text Goes Here\""])

    @decorators.idempotent_id('521ae1b0-f115-4d3c-8bcd-e54ef53bb2b9')
    def test_create_recordset_type_SVCB(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'SVCB record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_type_SVCB test.')

        ipv61 = '2001:db8:3333:4444:5555:6666:7777:8888'
        ipv6hint = f'{ipv61},2001:db8:3333:4444:cccc:dddd:eeee:ffff'
        ipv4hint = "1.2.3.4,9.8.7.6"
        alpn = "h3,h2,http/1.1"
        port = "888"
        target = "sample.example.org."
        doh = '/dns-query{?dns}'
        self._test_create_recordset_type(
            None,
            "SVCB",
            [f"1 {target} alpn={alpn} ipv4hint={ipv4hint}"
             f" ipv6hint={ipv6hint} port={port} dohpath={doh}"]
        )

    @decorators.idempotent_id('ebb3c1e2-fa33-4520-bba5-2888886d84f5')
    def test_create_recordset_type_HTTPS(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_type_HTTPS test.')

        ipv61 = '2001:db8:3333:4444:5555:6666:7777:8888'
        ipv6hint = f'{ipv61},2001:db8:3333:4444:cccc:dddd:eeee:ffff'
        ipv4hint = "1.2.3.4,9.8.7.6"
        alpn = "h3,h2,http/1.1"
        port = "888"
        target = "sample.example.org."
        self._test_create_recordset_type(
            "sample",
            "HTTPS",
            [f"1 {target} alpn={alpn} ipv4hint={ipv4hint}"
             f" ipv6hint={ipv6hint} port={port}"]
        )

    @decorators.idempotent_id('64f3a41b-065c-47f8-8a73-2a0cd62ed196')
    def test_create_recordset_type_HTTPS_no_default_alpn(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping '
                'test_create_recordset_type_HTTPS_no_default_alpntest.')

        self._test_create_recordset_type(
            "sample", "HTTPS", ['1 sample.example.org. alpn=http/1.1 '
                                'no-default-alpn port=8000']
        )

    @decorators.idempotent_id('4d399592-5d37-43ba-99ac-e7665c2c1f8f')
    def test_create_recordset_type_HTTPS_mandatory(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_type_HTTPS_mandatory '
                'test.')

        ipv6 = "2001:db8:3333:4444:5555:6666:7777:8888"
        self._test_create_recordset_type(
            "sample",
            "HTTPS",
            [f'1 sample.example.org. ipv4hint=192.168.1.2 ipv6hint={ipv6}'
             f' alpn=h2 mandatory=alpn'
             ]
        )

    def _test_create_wildcard_recordset(self, name, type, records):
        if name is not None:
            recordset_name = name + "." + self.zone['name']

        else:
            recordset_name = "*." + self.zone['name']

        recordset_data = {
            'name': recordset_name,
            'type': type,
            'records': records,
        }

        LOG.info('Create a Recordset')
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])

        LOG.info('Ensure we respond with PENDING')
        self.assertEqual(const.PENDING, body['status'])

    @decorators.idempotent_id('69f002e5-6511-43d3-abae-7abdd45ae03e')
    def test_create_wildcard_recordset_A_at_APEX(self):
        self._test_create_wildcard_recordset(
            "*", "A", ["192.0.2.1", "192.0.2.2", "192.0.2.3"])

    @decorators.idempotent_id('d97ee452-0dc3-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_A_under_APEX(self):
        self._test_create_wildcard_recordset(
            "*.sub", "A", ["192.0.2.1", "192.0.2.2", "192.0.2.3"])

    @decorators.idempotent_id('1b3c1cc0-0dc4-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_AAAA_at_APEX(self):
        self._test_create_wildcard_recordset(
            "*", "AAAA", ["2001:db8::1", "2001:db8::1", "2001:db8::"])

    @decorators.idempotent_id('928e735e-0dc4-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_AAAA_under_APEX(self):
        self._test_create_wildcard_recordset(
            "*.sub", "AAAA", ["2001:db8::1", "2001:db8::1", "2001:db8::"])

    @decorators.idempotent_id('d96138f2-0dc4-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_MX_at_APEX(self):
        self._test_create_wildcard_recordset(
            "*", "MX", ["10 mail1.example.org.", "20 mail2.example.org."])

    @decorators.idempotent_id('ff273c94-0dc4-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_MX_under_APEX(self):
        self._test_create_wildcard_recordset(
            "*.sub", "MX", ["10 mail.example.org."])

    @decorators.idempotent_id('3097f16a-0dc5-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_SPF_at_APEX(self):
        self._test_create_wildcard_recordset(
            "*", "SPF", ["\"v=spf1; a -all\""])

    @decorators.idempotent_id('50b3f390-0dc5-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_SPF_under_APEX(self):
        self._test_create_wildcard_recordset(
            "*.sub", "SPF", ["\"v=spf1; a -all\""])

    @decorators.idempotent_id('5b73981c-0dc5-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_TXT_at_APEX(self):
        self._test_create_wildcard_recordset(
            "*", "TXT", ["\"Can you read me?\""])

    @decorators.idempotent_id('5b73981c-0dc5-11ee-8b75-201e8823901f')
    def test_create_wildcard_recordset_TXT_under_APEX(self):
        self._test_create_wildcard_recordset(
            "*.sub", "TXT", ["\"Can you read me?\""])

    @decorators.idempotent_id('5964f730-5546-46e6-9105-5030e9c492b2')
    def test_list_recordsets(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a Recordset')
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        recordset_id = body['id']
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], recordset_id)

        LOG.info('List zone recordsets')
        body = self.client.list_recordset(self.zone['id'])[1]

        self.assertGreater(len(body), 0)

        # Test RBAC
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_reader',
                                     'os_project_member'])

        self.check_list_show_RBAC_enforcement(
            'RecordsetClient', 'list_recordset', expected_allowed, True,
            self.zone['id'])

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_reader',
                                     'os_project_member'])

        self.check_list_IDs_RBAC_enforcement(
            'RecordsetClient', 'list_recordset',
            expected_allowed, [recordset_id], self.zone['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'RecordsetClient', 'list_recordset', expected_allowed,
            [recordset_id], self.zone['id'], headers=self.all_projects_header)
        self.check_list_IDs_RBAC_enforcement(
            'RecordsetClient', 'list_recordset',
            expected_allowed, [recordset_id], self.zone['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('84c13cb2-9020-4c1e-aeb0-c348d9a70caa')
    def test_show_recordsets(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a Recordset')
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        recordset_id = body['id']
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], recordset_id)

        LOG.info('Re-Fetch the Recordset')
        record = self.client.show_recordset(self.zone['id'], recordset_id)[1]

        LOG.info('Ensure the fetched response matches the expected one')
        self.assertExpected(body, record, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'RecordsetClient', 'show_recordset', expected_allowed, True,
            self.zone['id'], recordset_id)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'RecordsetClient', 'show_recordset', expected_allowed, True,
            self.zone['id'], recordset_id, headers=self.all_projects_header)
        self.check_list_show_RBAC_enforcement(
            'RecordsetClient', 'show_recordset', expected_allowed, True,
            self.zone['id'], recordset_id,
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('855399c1-8806-4ae5-aa31-cb8a6f35e218')
    def test_delete_recordset(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a Recordset')
        record = self.client.create_recordset(
            self.zone['id'], recordset_data)[1]
        recordset_id = record['id']
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], recordset_id)

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'delete_recordset', expected_allowed, True,
            self.zone['id'], recordset_id)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'delete_recordset', expected_allowed, False,
            self.zone['id'], recordset_id, headers=self.all_projects_header)

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'delete_recordset', expected_allowed, False,
            self.zone['id'], recordset_id,
            headers={'x-auth-sudo-project-id': self.client.project_id})

        LOG.info('Delete a Recordset')
        self.client.delete_recordset(self.zone['id'], recordset_id)

        LOG.info('Ensure successful deletion of Recordset')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.show_recordset(self.zone['id'], recordset_id))

    @decorators.idempotent_id('8d41c85f-09f9-48be-a202-92d1bdf5c796')
    def test_update_recordset(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a recordset')
        record = self.client.create_recordset(
            self.zone['id'], recordset_data)[1]
        recordset_id = record['id']
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], recordset_id)

        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'], name=record['name'])

        LOG.info('Update the recordset')
        update = self.client.update_recordset(self.zone['id'],
            recordset_id, recordset_data)[1]

        self.assertEqual(record['name'], update['name'])
        self.assertNotEqual(record['records'], update['records'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'update_recordset', expected_allowed, True,
            self.zone['id'], recordset_id, recordset_data)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'update_recordset', expected_allowed, False,
            self.zone['id'], recordset_id, recordset_data,
            headers=self.all_projects_header)
        self.check_CUD_RBAC_enforcement(
            'RecordsetClient', 'update_recordset', expected_allowed, False,
            self.zone['id'], recordset_id, recordset_data,
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('60904cc5-148b-4e3b-a0c6-35656dc8d44c')
    def test_update_recordset_one_field(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a recordset')
        record = self.client.create_recordset(
            self.zone['id'], recordset_data)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], record['id'])

        recordset_data = {
            'ttl': dns_data_utils.rand_ttl(start=record['ttl'] + 1)
        }

        LOG.info('Update the recordset')
        update = self.client.update_recordset(self.zone['id'],
            record['id'], recordset_data)[1]

        self.assertEqual(record['name'], update['name'])
        self.assertEqual(record['records'], update['records'])
        self.assertEqual(record['description'], update['description'])
        self.assertNotEqual(record['ttl'], update['ttl'])

    @decorators.idempotent_id('3f3575a0-a28b-11eb-ab42-74e5f9e2a801')
    def test_show_recordsets_impersonate_another_project(self):

        LOG.info('Create a Recordset')
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])
        self.assertEqual(const.PENDING, body['status'],
                         'Failed, expected status is PENDING')
        LOG.info('Wait until the recordset is active')
        waiters.wait_for_recordset_status(
            self.client, self.zone['id'],
            body['id'], const.ACTIVE)

        LOG.info('Re-Fetch the Recordset as Alt tenant with '
                 '"x-auth-sudo-project-id" HTTP header included in request. '
                 'Expected: 403')
        self.assertRaises(
            lib_exc.Forbidden, lambda: self.alt_client.show_recordset(
                self.zone['id'], body['id'], headers={
                    'x-auth-sudo-project-id': body['project_id']}))

        LOG.info('Re-Fetch the Recordset as Admin tenant without '
                 '"x-auth-sudo-project-id" HTTP header. Expected: 404')
        self.assertRaises(lib_exc.NotFound,
                          lambda: self.admin_client.show_recordset(
                              self.zone['id'], body['id']))

        record = self.admin_client.show_recordset(
            self.zone['id'], body['id'],
            headers={'x-auth-sudo-project-id': body['project_id']})[1]

        LOG.info('Ensure the fetched response matches the expected one')
        self.assertExpected(body, record,
                            self.excluded_keys + ['action', 'status'])

    @decorators.idempotent_id('9f364a64-a2aa-11eb-aad4-74e5f9e2a801')
    def test_admin_list_all_recordsets_for_a_project(self):

        LOG.info('Create a Recordset as Primary tenant')
        recordset_data_primary_1 = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])
        body_pr_1 = self.client.create_recordset(
            self.zone['id'], recordset_data_primary_1)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body_pr_1['id'])
        self.assertEqual(const.PENDING, body_pr_1['status'],
                         'Failed, expected status is PENDING')
        recordset_data_primary_2 = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])
        body_pr_2 = self.client.create_recordset(
            self.zone['id'], recordset_data_primary_2)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body_pr_2['id'])
        self.assertEqual(const.PENDING, body_pr_2['status'],
                         'Failed, expected status is PENDING')

        LOG.info('Re-Fetch Recordsets as Alt tenant for a Primary project. '
                 'Expected: 404')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.alt_client.list_recordset(
                self.zone['id']))

        LOG.info('Re-Fetch Recordsets as Alt tenant for a Primary project '
                 'using "x-auth-all-projects" HTTP header. Expected: 403')
        self.assertRaises(lib_exc.Forbidden,
            lambda: self.alt_client.list_recordset(
                self.zone['id'],
                headers=self.all_projects_header))

        LOG.info('Re-Fetch Recordsets as Admin tenant for a Primary project '
                 'using "x-auth-all-projects" HTTP header.')
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        primary_recordsets_ids = [
            item['id'] for item in self.admin_client.list_recordset(
                self.zone['id'],
                headers=self.all_projects_header,
                params={'limit': 1000})[1]['recordsets']]

        for recordset_id in [body_pr_1['id'], body_pr_2['id']]:
            self.assertIn(
                recordset_id, primary_recordsets_ids,
                'Failed, recordset ID:{} was not found in listed '
                'recordsets: {}'.format(recordset_id, primary_recordsets_ids))

    @decorators.idempotent_id('48013b7c-f526-11eb-b04f-74e5f9e2a801')
    def test_create_A_recordset_multiply_ips(self):
        LOG.info('Create A type Recordset using a list of random IPs')
        recordset_data = dns_data_utils.rand_a_recordset(
            zone_name=self.zone['name'],
            ips=[dns_data_utils.rand_ip() for _ in range(10)])
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])
        LOG.info('Ensure we respond with PENDING')
        self.assertEqual(const.PENDING, body['status'])
        LOG.info('Wait until the recordset is active')
        waiters.wait_for_recordset_status(
            self.client, self.zone['id'],
            body['id'], const.ACTIVE)

    @decorators.idempotent_id('f15e583e-e479-11eb-8e5a-74e5f9e2a801')
    def test_delete_zone_with_existing_recordset(self):

        LOG.info('Create a Zone')
        zone_name = dns_data_utils.rand_zone_name(name="TestZone",
                                              suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a Recordset')
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        record = self.client.create_recordset(
            zone['id'], recordset_data, wait_until=const.ACTIVE)[1]

        LOG.info("Delete a Zone and wait till it's done")
        body = self.zones_client.delete_zone(zone['id'])[1]
        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual(const.DELETE, body['action'])
        self.assertEqual(const.PENDING, body['status'])

        LOG.info('Ensure successful deletion of Zone')
        waiters.wait_for_zone_404(self.zones_client, zone['id'])

        LOG.info('Ensure successful deletion of Recordset')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.show_recordset(zone['id'], record['id']))


class RecordsetsNegativeTest(BaseRecordsetsTest):

    credentials = ["admin", "primary", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(RecordsetsNegativeTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(RecordsetsNegativeTest, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.RecordsetClient()
        cls.alt_client = cls.os_alt.dns_v2.RecordsetClient()

    def _test_create_recordset_invalid(self, name, type, records):
        if name is not None:
            recordset_name = name + "." + self.zone['name']

        else:
            recordset_name = self.zone['name']

        recordset_data = {
            'name': recordset_name,
            'type': type,
            'records': records,
        }

        LOG.info('Attempt to create a invalid Recordset')
        self.assertRaises(lib_exc.BadRequest,
            lambda: self.client.create_recordset(
                self.zone['id'], recordset_data))

    @decorators.idempotent_id('98c94f8c-217a-4056-b996-b1f856d0753e')
    def test_create_recordset_invalid_CNAME_multiple(self):
        self._test_create_recordset_invalid(
            'www', 'CNAME', ["target1.example.org.", "target2.example.org."])

    @decorators.idempotent_id('43385ce8-0dd3-11ee-8b75-201e8823901f')
    def test_create_recordset_invalid_CNAME_at_Apex(self):
        self._test_create_recordset_invalid(
            None, 'CNAME', ["target1.example.org."])

    @decorators.idempotent_id('b6dad57e-5ce9-4fa5-8d66-aebbcd23b4ad')
    def test_get_nonexistent_recordset(self):
        LOG.info('Attempt to get an invalid Recordset')
        with self.assertRaisesDns(
                lib_exc.NotFound, 'recordset_not_found', 404):
            self.client.show_recordset(self.zone['id'],
                                       data_utils.rand_uuid())

    @decorators.idempotent_id('93d744a8-0dfd-4650-bcef-1e6ad632ad72')
    def test_get_nonexistent_recordset_invalid_id(self):
        LOG.info('Attempt to get an invalid Recordset')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_uuid', 400):
            self.client.show_recordset(self.zone['id'], 'invalid')

    @decorators.idempotent_id('da08f19a-7f10-47cc-8b41-994507190812')
    def test_update_nonexistent_recordset(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            'A', self.zone['name'])

        LOG.info('Attempt to update an invalid Recordset')
        with self.assertRaisesDns(
                lib_exc.NotFound, 'recordset_not_found', 404):
            self.client.update_recordset(
                self.zone['id'], data_utils.rand_uuid(), recordset_data)

    @decorators.idempotent_id('158340a1-3f69-4aaa-9968-956190563768')
    def test_update_nonexistent_recordset_invalid_id(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            'A', self.zone['name'])

        LOG.info('Attempt to update an invalid Recordset')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_uuid', 400):
            self.client.update_recordset(
                self.zone['id'], 'invalid', recordset_data)

    @decorators.idempotent_id('64bd94d4-54bd-4bee-b6fd-92ede063234e')
    def test_delete_nonexistent_recordset(self):
        LOG.info('Attempt to delete an invalid Recordset')
        with self.assertRaisesDns(
                lib_exc.NotFound, 'recordset_not_found', 404):
            self.client.delete_recordset(
                self.zone['id'], data_utils.rand_uuid())

    @decorators.idempotent_id('5948b599-a332-4dcb-840b-afc825075ba3')
    def test_delete_nonexistent_recordset_invalid_id(self):
        LOG.info('Attempt to get an invalid Recordset')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_uuid', 400):
            self.client.delete_recordset(self.zone['id'], 'invalid')

    @decorators.idempotent_id('64e01dc4-a2a8-11eb-aad4-74e5f9e2a801')
    def test_show_recordsets_invalid_ids(self):

        LOG.info('Create a Recordset')
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])
        resp, body = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], body['id'])
        self.assertEqual(const.PENDING, body['status'],
                         'Failed, expected status is PENDING')
        LOG.info('Wait until the recordset is active')
        waiters.wait_for_recordset_status(
            self.client, self.zone['id'],
            body['id'], const.ACTIVE)

        LOG.info('Ensure 404 NotFound status code is received if '
                 'recordset ID is invalid.')
        self.assertRaises(
            lib_exc.NotFound, lambda: self.client.show_recordset(
                zone_uuid=self.zone['id'],
                recordset_uuid=data_utils.rand_uuid()))

        LOG.info('Ensure 404 NotFound status code is received if '
                 'zone ID is invalid.')
        self.assertRaises(
            lib_exc.NotFound, lambda: self.client.show_recordset(
                zone_uuid=data_utils.rand_uuid(),
                recordset_uuid=body['id']))

    @decorators.idempotent_id('c1d9f046-a2b1-11eb-aad4-74e5f9e2a801')
    def test_create_recordset_for_other_tenant(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        LOG.info('Create a Recordset as Alt tenant for a zone created by '
                 'Primary tenant. Expected: 404 NotFound')
        self.assertRaises(
            lib_exc.NotFound, lambda: self.alt_client.create_recordset(
                self.zone['id'], recordset_data))

    @decorators.idempotent_id('f236a24a-a3c6-4a0c-a54c-ebf8a312b908')
    def test_create_recordset_invalid_HTTPS_alpn(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_HTTPS_alpn '
                'test.')

        self._test_create_recordset_invalid(
            None, 'HTTPS', ["1 sample.example.org. alpn=foo"])

    @decorators.idempotent_id('d680ef3a-4406-47c5-a994-483138c5e975')
    def test_create_recordset_invalid_HTTPS_port(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_HTTPS_port '
                'test.')

        self._test_create_recordset_invalid(
            None, 'HTTPS', ["1 sample.example.org. port=foo"])

    @decorators.idempotent_id('a33fc327-8d37-4582-9d2b-be83f6ee38d3')
    def test_create_recordset_invalid_HTTPS_ech(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'HTTPS record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_HTTPS_ech '
                'test.')

        self._test_create_recordset_invalid(
            None, 'HTTPS', ["1 sample.example.org. port=8888 ech=foo//"])

    @decorators.idempotent_id('1869737f-1a1b-4712-93da-2d680cd45cd8')
    def test_create_recordset_invalid_SVCB_mandatory(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'SVCB record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_SVCB_mandatory '
                'test.')

        ipv6hint = "2001:db8:3333:4444:5555:6666:7777:8888"
        self._test_create_recordset_invalid(
            None, 'SVCB', [
                f"1 sample.example.org."
                f" ipv6hint={ipv6hint} mandatory=alpn,ipv4hint"
            ]
        )

    @decorators.idempotent_id('5bae6c22-7a37-48a5-b6b3-089d83a8f2ce')
    def test_create_recordset_invalid_SVCB_ipv4hint(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'SVCB record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_SVCB_ipv4hint '
                'test.')

        self._test_create_recordset_invalid(
            None, 'SVCB',
            ['1 sample.example.org. ipv4hint=foo,192.168.1.2'])

    @decorators.idempotent_id('72dce8ca-f2fa-4054-b2b7-7bccfe3a02c1')
    def test_create_recordset_invalid_SVCB_ipv6hint(self):
        if not versionutils.is_compatible('2.2', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'SVCB record tests require Designate API version 2.2 or '
                'newer. Skipping test_create_recordset_invalid_SVCB_ipv6hint'
                'test.')

        self._test_create_recordset_invalid(
            None, 'SVCB', ['1 sample.example.org. ipv6hint=foo']
        )


class RootRecordsetsTests(BaseRecordsetsTest):
    credentials = ["admin", "primary", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(RootRecordsetsTests, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(RootRecordsetsTests, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.RecordsetClient()

    @classmethod
    def skip_checks(cls):
        super(RootRecordsetsTests, cls).skip_checks()

        if not CONF.dns_feature_enabled.api_v2_root_recordsets:
            skip_msg = ("%s skipped as designate V2 recordsets API is not "
                        "available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.idempotent_id('48a081b9-4474-4da0-9b1a-6359a80456ce')
    def test_list_zones_recordsets(self):
        LOG.info('List recordsets')
        body = self.client.list_zones_recordsets()[1]

        self.assertGreater(len(body['recordsets']), 0)

    @decorators.idempotent_id('65ec0495-81d9-4cfb-8007-9d93b32ae883')
    def test_get_single_zones_recordsets(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'], records=['10.1.0.2'])

        LOG.info('Create a Recordset')
        resp, zone_recordset = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], zone_recordset['id'])

        self.client.show_zones_recordset(zone_recordset['id'])

    @decorators.idempotent_id('a8e41020-65be-453b-a8c1-2497d539c345')
    def test_list_filter_zones_recordsets(self):
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'], records=['10.0.1.2'])

        LOG.info('Create a Recordset')
        resp, zone_recordset = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], zone_recordset['id'])

        LOG.info('Create another zone')
        zone_name = dns_data_utils.rand_zone_name(name="list-filter",
                                              suffix=self.tld_name)
        zone2 = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone2['id'])

        LOG.info('Create another Recordset')
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone2['name'],
            records=['10.0.1.3'])
        resp, zone_recordset2 = self.client.create_recordset(
            zone2['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], zone_recordset2['id'])

        LOG.info('List recordsets')
        body = self.client.list_zones_recordsets(params={"data": "10.0.*"})[1]

        recordsets = body['recordsets']

        ids = [r['id'] for r in recordsets]
        self.assertIn(zone_recordset['id'], ids)
        self.assertIn(zone_recordset2['id'], ids)
        # Ensure that every rrset has a record with the filtered data
        for r in recordsets:
            one_record_has_data = False
            for record in r['records']:
                if record.startswith('10.0.'):
                    one_record_has_data = True
            self.assertTrue(one_record_has_data)

    @decorators.idempotent_id('7f4970bf-9aeb-4a3c-9afd-02f5a7178d35')
    def test_list_zones_recordsets_zone_names(self):
        LOG.info('List recordsets')
        zone_name = dns_data_utils.rand_zone_name(name="zone_names",
                                              suffix=self.tld_name)
        alt_zone = self.zones_client.create_zone(
            name=zone_name, wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.zones_client,
                        alt_zone['id'])

        body = self.client.list_zones_recordsets()[1]

        recordsets = body['recordsets']
        zone_names = set()
        for r in recordsets:
            zone_names.add(r['zone_name'])

        self.assertGreaterEqual(len(zone_names), 2)


class RecordsetOwnershipTest(BaseRecordsetsTest):

    credentials = ["primary", "alt", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(RecordsetOwnershipTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(RecordsetOwnershipTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.client = cls.os_primary.dns_v2.RecordsetClient()
        cls.alt_client = cls.os_alt.dns_v2.RecordsetClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()

    def _create_client_recordset(self, clients_list):
        """Create a zone and asoociated recordset using given credentials
        :param clients_list: supported credentials are: 'primary' and 'alt'.
        :return: dictionary of created recordsets.
        """
        recordsets_created = {}
        for client in clients_list:
            if client == 'primary':
                # Create a zone and wait till it's ACTIVE
                zone_name = dns_data_utils.rand_zone_name(name="primary",
                                                      suffix=self.tld_name)
                zone = self.zones_client.create_zone(name=zone_name)[1]
                self.addCleanup(self.wait_zone_delete,
                                self.zones_client,
                                zone['id'])
                waiters.wait_for_zone_status(
                    self.zones_client, zone['id'], const.ACTIVE)

                # Create a recordset and wait till it's ACTIVE
                recordset_data = dns_data_utils.rand_recordset_data(
                    record_type='A', zone_name=zone['name'])
                resp, body = self.client.create_recordset(
                    zone['id'], recordset_data)

                self.addCleanup(
                    self.wait_recordset_delete, self.client,
                    self.zone['id'], body['id'])
                self.assertEqual(const.PENDING, body['status'],
                                 'Failed, expected status is PENDING')

                LOG.info('Wait until the recordset is active')
                waiters.wait_for_recordset_status(
                    self.client, zone['id'],
                    body['id'], const.ACTIVE)

                # Add "project_id" into the recordset_data
                recordset_data['project_id'] = zone['project_id']
                recordsets_created['primary'] = recordset_data

            if client == 'alt':
                # Create a zone and wait till it's ACTIVE
                zone_name = dns_data_utils.rand_zone_name(name="alt",
                                                      suffix=self.tld_name)
                alt_zone = self.alt_zone_client.create_zone(name=zone_name)[1]
                self.addCleanup(self.wait_zone_delete,
                                self.alt_zone_client,
                                alt_zone['id'])
                waiters.wait_for_zone_status(
                    self.alt_zone_client, alt_zone['id'], const.ACTIVE)

                # Create a recordset and wait till it's ACTIVE
                recordset_data = dns_data_utils.rand_recordset_data(
                    record_type='A', zone_name=alt_zone['name'])
                resp, body = self.alt_client.create_recordset(
                    alt_zone['id'], recordset_data)

                self.addCleanup(
                    self.wait_recordset_delete, self.client,
                    self.zone['id'], body['id'])
                self.assertEqual(const.PENDING, body['status'],
                                 'Failed, expected status is PENDING')

                LOG.info('Wait until the recordset is active')
                waiters.wait_for_recordset_status(
                    self.alt_client, alt_zone['id'],
                    body['id'], const.ACTIVE)

                # Add "project_id" into the recordset_data
                recordset_data['project_id'] = alt_zone['project_id']
                recordsets_created['alt'] = recordset_data

        LOG.info('Created resordsets are {}:'.format(recordsets_created))
        return recordsets_created

    @decorators.idempotent_id('9c0f58ad-1b31-4899-b184-5380720604e5')
    def test_no_create_recordset_by_alt_tenant(self):
        # try with name=A123456.zone.com.
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])
        resp, rrset = self.client.create_recordset(
            self.zone['id'], recordset_data)
        self.addCleanup(
            self.wait_recordset_delete, self.client,
            self.zone['id'], rrset['id'])
        self.assertRaises(
            lib_exc.RestClientException,
            lambda: self.alt_client.create_recordset(
                self.zone['id'], recordset_data)
        )

    @decorators.idempotent_id('d4a9aad9-c778-429b-9a0c-4cd2b61a0a01')
    def test_no_create_super_recordsets(self):
        zone_name = dns_data_utils.rand_zone_name(suffix=self.tld_name)

        LOG.info('Create a zone as a default user')
        zone = self.zones_client.create_zone(name='a.b.' + zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        rrset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone_name)

        LOG.info('Create a zone as an alt user with existing superdomain')
        self.assertRaises(
            lib_exc.NotFound,
            self.alt_client.create_recordset,
            self.zone['id'], rrset_data)

    @decorators.idempotent_id('3dbe244d-fa85-4afc-869b-0306388d8746')
    def test_no_create_recordset_via_alt_domain(self):
        zone_name = dns_data_utils.rand_zone_name(name="alt-domain",
                                              suffix=self.tld_name)
        alt_zone = self.alt_zone_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.alt_zone_client,
                        alt_zone['id'])

        # alt attempts to create record with name A12345.{zone}
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=self.zone['name'])

        self.assertRaises(
            lib_exc.RestClientException,
            lambda: self.alt_client.create_recordset(
                self.zone['id'],
                recordset_data
            )
        )
        self.assertRaises(
            lib_exc.RestClientException,
            lambda: self.alt_client.create_recordset(
                alt_zone['id'],
                recordset_data
            )
        )

    @decorators.idempotent_id('4d0ff972-7c19-11eb-b331-74e5f9e2a801')
    def test_list_all_recordsets_for_project(self):
        # Create recordsets using "primary" and "alt" credentials.
        # Execute "list_owned_recordsets" API to list "primary" recordsets.
        # Validate that the only "project_id" retrieved within the API is
        # a "primary" project.
        primary_project_id = self._create_client_recordset(
            ['primary', 'alt'])['primary']['project_id']
        recordsets = self.client.list_owned_recordsets()
        LOG.info('Received by API recordsets are {} '.format(recordsets))
        project_ids_api = set([item['project_id'] for item in recordsets])
        self.assertEqual(
            {primary_project_id}, project_ids_api,
            'Failed, unique project_ids {} are not as expected {}'.format(
                project_ids_api, primary_project_id))

    @decorators.idempotent_id('bc0af248-7b4f-11eb-98a5-74e5f9e2a801')
    def test_list_all_projects_recordsets(self):
        # Create recordsets using "primary" and "alt" credentials.
        # Execute "list_owned_recordsets" API using admin client to list
        # recordsets for all projects.
        # Validate that project_ids of: "primary" and "alt" projects
        # are both listed in received API response.
        project_ids_used = [
            item['project_id'] for item in self._create_client_recordset(
                ['primary', 'alt']).values()]
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        recordsets = self.admin_client.list_owned_recordsets(
            headers=self.all_projects_header, params={'limit': 1000})
        LOG.info('Received by API recordsets are {} '.format(recordsets))
        project_ids_api = set([item['project_id'] for item in recordsets])
        for prj_id in project_ids_used:
            self.assertIn(
                prj_id, project_ids_api,
                'Failed, project_id:{} is missing in received recordsets'
                ' for all projects {} '.format(prj_id, project_ids_api))

    @decorators.idempotent_id('910eb17e-7c3a-11eb-a40b-74e5f9e2a801')
    def test_list_recordsets_impersonate_project(self):
        # Create recordsets using "primary" and "alt" credentials.
        # Use admin client to impersonate "primary" project.
        # Validate that received recordsets are all associated with
        # expected("primary") project only.
        primary_project_id = self._create_client_recordset(
            ['primary', 'alt'])['primary']['project_id']
        recordsets = self.admin_client.list_owned_recordsets(
            headers={'x-auth-sudo-project-id': primary_project_id})
        LOG.info('Received by API recordsets are {} '.format(recordsets))
        project_ids_api = set([item['project_id'] for item in recordsets])
        self.assertEqual(
            {primary_project_id}, project_ids_api,
            'Failed, unique project_ids {} are not as expected {}'.format(
                project_ids_api, primary_project_id))


class AdminManagedRecordsetTest(BaseRecordsetsTest):

    credentials = ["primary", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(AdminManagedRecordsetTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(AdminManagedRecordsetTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.client = cls.os_primary.dns_v2.RecordsetClient()

    @decorators.idempotent_id('84164ff4-8e68-11ec-983f-201e8823901f')
    def test_admin_updates_soa_and_ns_recordsets(self):
        # HTTP headers to be used in the test
        sudo_header = {'X-Auth-All-Projects': True}
        managed_records_header = {'X-Designate-Edit-Managed-Records': True}
        sudo_managed_headers = sudo_header.copy()
        sudo_managed_headers.update(managed_records_header)

        LOG.info('Primary user creates a Zone')
        zone_name = dns_data_utils.rand_zone_name(name="update_soa_ns",
                                              suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name,
            description='Zone for "managed recordsets update" test',
            wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        recordsets = self.admin_client.list_recordset(
            zone['id'], headers=sudo_header)[1]['recordsets']

        LOG.info('As Admin try to update SOA and NS recordsets,'
                 ' Expected not allowed')
        for recordset in recordsets:
            if recordset['type'] == 'NS':
                self.assertRaisesDns(
                    lib_exc.BadRequest, 'bad_request', 400,
                    self.admin_client.update_recordset,
                    zone['id'], recordset['id'],
                    recordset_data=dns_data_utils.rand_ns_records(),
                    headers=sudo_managed_headers, extra_headers=True)

            if recordset['type'] == 'SOA':
                self.assertRaisesDns(
                    lib_exc.BadRequest, 'bad_request', 400,
                    self.admin_client.update_recordset,
                    zone['id'], recordset['id'],
                    recordset_data=dns_data_utils.rand_soa_recordset(
                        zone['name']),
                    headers=sudo_managed_headers, extra_headers=True)


class RecordsetsManagedRecordsNegativeTest(BaseRecordsetsTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_clients(cls):
        super(RecordsetsManagedRecordsNegativeTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()

    @decorators.idempotent_id('083fa738-bb1b-11ec-b581-201e8823901f')
    def test_delete_ns_record_not_permitted(self):
        LOG.info('Get NS type recordset ID')
        recordsets = self.recordset_client.list_recordset(
            self.zone['id'])[1]['recordsets']
        for recordset in recordsets:
            if recordset['type'] == 'NS':
                ns_record_id = recordset['id']
                break

        LOG.info('Primary user tries to delete NS Recordset')
        self.assertRaises(
            lib_exc.Forbidden,
            self.recordset_client.delete_recordset,
            self.zone['id'], ns_record_id, headers=self.managed_records)

    @decorators.idempotent_id('1e78a742-66ee-11ec-8dc3-201e8823901f')
    def test_create_soa_record_not_permitted(self):
        soa_record = ("s1.devstack.org. admin.example.net. 1510721487 3510"
                      " 600 86400 3600")
        LOG.info('Primary tries to create a Recordset on '
                 'the existing zone')
        self.assertRaises(
            lib_exc.BadRequest,
            self.recordset_client.create_recordset,
            self.zone['id'], soa_record)
        LOG.info('Admin tries to create a Recordset on the existing zone')
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_client.create_recordset,
            self.zone['id'], soa_record)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import sys
import linecache

from prettytable import PrettyTable

PEP8_LINE = r'^((?P<file>.*):(?P<line>\d*):(?P<col>\d*):) ' \
            r'(?P<error>(?P<error_code>\w\d{1,3})(?P<error_desc>.*$))'

HTML = True


def main():

    raw_errors = []

    max_filename_len = 0
    for line in sys.stdin:
        m = re.match(PEP8_LINE, line)
        if m:
            m = m.groupdict()
            raw_errors.append(m)
            if len(m['file']) > max_filename_len:
                max_filename_len = len(m['file'])
        else:
            print(line)

    if len(raw_errors) > 0:

        print('Flake8 Results')

        ct = PrettyTable([
            "File",
            "Line",
            "Column",
            "Error Code",
            "Error Message",
            "Code"
        ])

        ct.align["File"] = "l"
        ct.align["Error Message"] = "l"
        ct.align["Code"] = "l"

        for line in raw_errors:
            ct.add_row(format_dict(line))

        print(ct)

        with open('flake8_results.html', 'w') as f:
            f.write('<html><head><style type="text/css">table a:link{color:#666;font-weight:700;text-decoration:none}table a:visited{color:#999;font-weight:700;text-decoration:none}table a:active,table a:hover{color:#bd5a35;text-decoration:underline}table{font-family:Arial,Helvetica,sans-serif;color:#666;font-size:12px;text-shadow:1px 1px 0 #fff;background:#eaebec;margin:20px;border:1px solid #ccc;-moz-border-radius:3px;-webkit-border-radius:3px;border-radius:3px;-moz-box-shadow:0 1px 2px #d1d1d1;-webkit-box-shadow:0 1px 2px #d1d1d1;box-shadow:0 1px 2px #d1d1d1}table th{padding:21px 25px 22px;border-top:1px solid #fafafa;border-bottom:1px solid #e0e0e0;background:#ededed;background:-webkit-gradient(linear,left top,left bottom,from(#ededed),to(#ebebeb));background:-moz-linear-gradient(top,#ededed,#ebebeb)}table th:first-child{text-align:left;padding-left:20px}table tr:first-child th:first-child{-moz-border-radius-topleft:3px;-webkit-border-top-left-radius:3px;border-top-left-radius:3px}table tr:first-child th:last-child{-moz-border-radius-topright:3px;-webkit-border-top-right-radius:3px;border-top-right-radius:3px}table tr{text-align:left;padding-left:20px}table td:first-child{text-align:left;padding-left:20px;border-left:0}table td{padding:18px;border-top:1px solid #fff;border-bottom:1px solid #e0e0e0;border-left:1px solid #e0e0e0;background:#fafafa;background:-webkit-gradient(linear,left top,left bottom,from(#fbfbfb),to(#fafafa));background:-moz-linear-gradient(top,#fbfbfb,#fafafa)}table tr.even td{background:#f6f6f6;background:-webkit-gradient(linear,left top,left bottom,from(#f8f8f8),to(#f6f6f6));background:-moz-linear-gradient(top,#f8f8f8,#f6f6f6)}table tr:last-child td{border-bottom:0}table tr:last-child td:first-child{-moz-border-radius-bottomleft:3px;-webkit-border-bottom-left-radius:3px;border-bottom-left-radius:3px}table tr:last-child td:last-child{-moz-border-radius-bottomright:3px;-webkit-border-bottom-right-radius:3px;border-bottom-right-radius:3px}table tr:hover td{background:#f2f2f2;background:-webkit-gradient(linear,left top,left bottom,from(#f2f2f2),to(#f0f0f0));background:-moz-linear-gradient(top,#f2f2f2,#f0f0f0)}</style></head><body>%s</body</html>' % ct.get_html_string(attributes = {"cellspacing": 0})) # noqa


def format_dict(raw):
    output = []
    if raw['file'].startswith('./'):
        output.append(raw['file'][2:])
    else:
        output.append(raw['file'])

    output.append(raw['line'])
    output.append(raw['col'])
    output.append(raw['error_code'])

    output.append(raw['error_desc'].lstrip())

    code_string = linecache.getline(
        output[0],
        int(raw['line'])).lstrip().rstrip()

    output.append(code_string)

    return output

if __name__ == '__main__':
    main()



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.services.dns.v2.json import base
from tempest import config

CONF = config.CONF


class PtrClient(base.DnsClientV2Base):

    @base.handle_errors
    def set_ptr_record(self, floatingip_id, ptr_name=None,
                       ttl=None, description=None, headers=None,
                       tld=None):
        """Set a PTR record for the given FloatingIP

        :param floatingip_id: valid UUID of floating IP to be used.
        :param ptr_name PTR record name or random if not provided.
        :param ttl TTL or random valid value if not provided.
        :param description Description or random if not provided.
        :param headers (dict): The headers to use for the request.
        :param tld, the TLD to be used in ptrdname generated value.
        :return: created PTR dictionary.
        """
        ptr = {
            'ptrdname': ptr_name or dns_data_utils.rand_domain_name(tld),
            'ttl': ttl or dns_data_utils.rand_ttl(),
            'description': description or data_utils.rand_name('test-ptr')}

        return self._update_request(
            resource='reverse/floatingips/{}'.format(CONF.identity.region),
            uuid=floatingip_id, data=ptr, headers=headers,
            uuid_prefix_char=':')[1]

    @base.handle_errors
    def show_ptr_record(self, floatingip_id, headers=None):
        """Show PTR record for the given FloatingIP

        :param floatingip_id: valid UUID of floating IP to show.
        :param headers (dict): The headers to use for the request.
        :return: Shown PTR dictionary.
        """
        return self._show_request(
            resource='reverse/floatingips/{}'.format(CONF.identity.region),
            uuid=floatingip_id, headers=headers, uuid_prefix_char=':')[1]

    @base.handle_errors
    def list_ptr_records(self, headers=None):
        """List PTR records for the given FloatingIP

        :param headers (dict): The headers to use for the request.
        :return: List of PTR records.
        """
        return self._list_request(
            'reverse/floatingips', headers=headers)[1]['floatingips']

    @base.handle_errors
    def unset_ptr_record(self, floatingip_id, headers=None):
        """Unset the PTR record for a given FloatingIP

        :param floatingip_id: valid UUID of floating IP to unset.
        :param headers (dict): The headers to use for the request.
        :return: Tuple (Response, Body)
        """
        data = {"ptrdname": None}
        resp, body = self._update_request(
            resource='reverse/floatingips/{}'.format(CONF.identity.region),
            uuid=floatingip_id, data=data, headers=headers,
            uuid_prefix_char=':')
        # Unset PTR should Return a HTTP 202
        self.expected_success(202, resp.status)
        return resp, body



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.v2.json import base


class ZoneImportsClient(base.DnsClientV2Base):

    @base.handle_errors
    def create_zone_import(self, zonefile_data=None,
                           wait_until=None, headers=None):
        """Create a zone import.
        :param zonefile_data: A tuple that represents zone data.
        :param wait_until: If not None, a waiter for appropriate status
                will be activated.
        :param headers (dict): The headers to use for the request.
        :return: Serialized imported zone as a dictionary.
        """
        if not headers:
            headers = {'Content-Type': 'text/dns'}
        zone_data = zonefile_data or dns_data_utils.rand_zonefile_data()
        resp, body = self._create_request(
            'zones/tasks/imports', zone_data, headers=headers)

        # Create Zone should Return a HTTP 202
        self.expected_success(202, resp.status)

        if wait_until:
            waiters.wait_for_zone_import_status(self, body['id'], wait_until)

        return resp, body

    @base.handle_errors
    def show_zone_import(self, uuid, params=None, headers=None):
        """Gets a specific zone import.
        :param uuid: Unique identifier of the imported zone in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized imported zone as a dictionary.
        """
        return self._show_request(
            'zones/tasks/imports', uuid, params=params, headers=headers)

    @base.handle_errors
    def list_zone_imports(self, params=None, headers=None):
        """Gets all the imported zones.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized imported zones as a list.
        """
        return self._list_request(
            'zones/tasks/imports', params=params, headers=headers)

    @base.handle_errors
    def delete_zone_import(self, uuid, params=None, headers=None):
        """Deletes a imported zone having the specified UUID.
        :param uuid: The unique identifier of the imported zone.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request(
            'zones/tasks/imports', uuid, params=params, headers=headers)

        # Delete Zone should Return a HTTP 204
        self.expected_success(204, resp.status)

        return resp, body



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from designate_tempest_plugin.common import constants as const
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ServiceStatusAdmin(base.BaseDnsV2Test):

    credentials = ["primary", "admin", "alt",
                   "project_reader", "project_member"]

    mandatory_services = ['central', 'mdns', 'worker', 'producer']
    service_status_fields = [
        'id', 'hostname', 'service_name', 'status', 'stats', 'capabilities',
        'heartbeated_at', 'created_at', 'updated_at', 'links']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ServiceStatusAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServiceStatusAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ServiceClient()

    @decorators.idempotent_id('bf277a76-8583-11eb-a557-74e5f9e2a801')
    def test_admin_list_service_statuses(self):

        services_statuses_tup = [
            (item['service_name'],
             item['status']) for item in self.admin_client.list_statuses()]
        LOG.info("Listed service tuples: (name,status)' are:{} ".format(
            services_statuses_tup))

        LOG.info('Make sure that all expected/mandatory services are '
                 'listed in API response.')

        for service in self.mandatory_services:
            self.assertIn(
                service, [item[0] for item in services_statuses_tup],
                "Failed, expected service: {} wasn't detected in API "
                "response".format(service))

        LOG.info('Make sure that all listed services are in UP status.')
        self.assertEqual(
            {const.UP}, set([item[1] for item in services_statuses_tup]),
            "Failed, not all listed services are in UP status, "
            "services: {}".format(services_statuses_tup))

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ServiceClient', 'list_statuses', expected_allowed, False)

    @decorators.idempotent_id('fce0f704-c0ae-11ec-8213-201e8823901f')
    def test_admin_show_service_status(self):

        LOG.info('List services and get the IDs of mandatory services only')
        services_ids = [
            service['id'] for service in self.admin_client.list_statuses()
            if service['service_name'] in self.mandatory_services]

        LOG.info('Ensure all service status fields presents in response')
        for id in services_ids:
            service_show = self.admin_client.show_statuses(id)
            self.assertEqual(
                sorted(self.service_status_fields), sorted(service_show))


class ServiceStatusNegative(base.BaseDnsV2Test):

    credentials = ["primary", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ServiceStatusNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServiceStatusNegative, cls).setup_clients()
        cls.primary_client = cls.os_primary.dns_v2.ServiceClient()
        cls.alt_client = cls.os_alt.dns_v2.ServiceClient()

    @decorators.idempotent_id('d4753f76-de43-11eb-91d1-74e5f9e2a801')
    def test_primary_is_forbidden_to_list_service_statuses(self):

        LOG.info('Try to "list service statuses" as Primary user')
        self.assertRaises(
            lib_exc.Forbidden, self.primary_client.list_statuses)

        headers = [{'x-auth-all-projects': True},
                   {'x-auth-sudo-project-id': self.alt_client.project_id}]
        for header in headers:
            LOG.info('Try to "list service statuses" using {} '
                     'HTTP header'.format(header))
            self.assertRaises(
                lib_exc.Forbidden, self.primary_client.list_statuses,
                headers=header)



# Copyright 2024 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import random

import testtools
import yaml
from oslo_log import log as logging
from tempest import config
import subprocess

from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import resources
from designate_tempest_plugin.services.dns.query.query_client import (
    SingleQueryClient)

CONF = config.CONF
LOG = logging.getLogger(__name__)


class DesignateManageTest(base.BaseDnsV2Test):
    credentials = ["admin", 'primary', 'system_admin']
    managed_resource = None

    @classmethod
    def skip_checks(cls) -> None:
        super().skip_checks()
        if CONF.dns_feature_enabled.designate_manage_path:
            cls.designate_manage_cmd = (
                CONF.dns_feature_enabled.designate_manage_path)
        else:
            raise cls.skipException('designate-manage path was not found. '
                                    'Skipping this test class')

    @classmethod
    def _run_designate_manage_command(cls,
                                      managed_resource: str,
                                      command: str,
                                      *args) -> str:
        """Runs the designate-manage command with the provided arguments.

        :param managed_resource: (str): The resource managed by the
         designate-manage command. For example: pool
        :param command: (str): The command to run. For example: update
        :return: The command output
        """
        managed_resource = managed_resource or cls.managed_resource

        commands_list = [cls.designate_manage_cmd, managed_resource]
        if command and isinstance(command, tuple):
            commands_list.extend(command)
        else:
            commands_list.append(command)
        if args and isinstance(args, tuple):
            commands_list.extend(args[0])
        try:
            output = subprocess.check_output(commands_list,
                    stderr=subprocess.STDOUT,
                    text=True)
            return output
        except subprocess.CalledProcessError as e:
            LOG.error(e.output)


def _get_pools_path(name: str) -> str:
    return os.path.join(resources.path, 'pools_yaml', name)


@testtools.skipUnless(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
class DesignateManagePoolTest(DesignateManageTest):
    managed_resource = 'pool'
    file_attributes_to_num_of_appearances = {  # per each Pool
        'also_notifies:': 1, 'attributes:': 1, 'description:': 2, 'id:': 1,
        'name:': 2, 'nameservers:': 1, 'ns_records:': 1, 'targets:': 1
    }
    MULTIPOOLS_FILE_PATH = "/etc/designate/multiple-pools.yaml"

    @classmethod
    def resource_setup(cls):
        testtools.skipUnless(os.path.exists(cls.MULTIPOOLS_FILE_PATH),
        f"multiple-pools configuration file {cls.MULTIPOOLS_FILE_PATH} was "
        "not found, skipping the test")
        cls._update_pools_file(cls.MULTIPOOLS_FILE_PATH)

    @classmethod
    def _update_pools_file(cls, pools_file_path):
        if not pools_file_path.startswith('/'):
            pools_file_path = _get_pools_path(name=pools_file_path)
        cls._run_designate_manage_pool_command(
            'update',
            '--file',
            pools_file_path,
            '--delete')

    def tearDown(self):
        super(DesignateManagePoolTest, self).tearDown()
        self._update_pools_file(self.MULTIPOOLS_FILE_PATH)

    @staticmethod
    def _load_config(filename) -> str:
        with open(filename) as stream:
            return yaml.safe_load(stream)

    @classmethod
    def _run_designate_manage_pool_command(cls, command: str, *args) -> str:
        return super(
            DesignateManagePoolTest, cls)._run_designate_manage_command(
            'pool', command, args)

    @decorators.idempotent_id('ed42f367-e5ba-40d7-a08d-366ad787d21d')
    def test_pool_show_config(self):
        self._update_pools_file(self.MULTIPOOLS_FILE_PATH)
        pool_config = self._run_designate_manage_pool_command(
            command='show_config').split('\n\n')[0]
        self.assertIn('BIND Pool', pool_config)

    @decorators.idempotent_id('ed42f367-e5ba-40d7-a08d-366ad787d21e')
    def test_pool_show_config_all(self):
        self._update_pools_file(pools_file_path='multiple-pools.yaml')
        pool_config = self._run_designate_manage_pool_command(
            'show_config', '--all').split('\n\n')[0]

        pool_config_list = pool_config.split('\n')

        for attribute in self.file_attributes_to_num_of_appearances:
            num_of_occurrences = sum(attribute in s for s in pool_config_list)
            file_attributes_to_num_of_appearances = {
                'also_notifies:': 2, 'attributes:': 2, 'description:': 4,
                'id:': 2, 'name:': 7, 'nameservers:': 2, 'ns_records:': 2,
                'targets:': 2
            }
            err_msg = (f'{attribute} was supposed to appear '
                       f'{file_attributes_to_num_of_appearances[attribute]} '
                       'times on the designate-manage output, but '
                       f'it appeared {num_of_occurrences} times.')
            self.assertEqual(
                file_attributes_to_num_of_appearances[attribute],
                num_of_occurrences, err_msg)

    @decorators.idempotent_id('ed42f367-e5ba-40d7-a08d-366ad787d220')
    def test_pool_update_multiple_pools_without_delete(self):

        # Updating to multiple-pools.yaml with --delete
        self._update_pools_file(pools_file_path='multiple-pools.yaml')

        # Updating to other-pools.yaml without --delete. There should be 5
        # pools after the update
        pools_yaml_path = _get_pools_path(name='other-pools.yaml')
        self._run_designate_manage_pool_command(
            'update',
            '--file',
            pools_yaml_path,
        )

        pool_config = self._run_designate_manage_pool_command(
            'show_config', '--all').split('\n\n')[0]

        pool_config_list = pool_config.split('\n')
        for attribute in self.file_attributes_to_num_of_appearances:
            num_of_occurrences = sum(attribute in s for s in pool_config_list)
            file_attributes_to_num_of_appearances = {
                'also_notifies:': 3, 'attributes:': 3, 'description:': 6,
                'id:': 3, 'name:': 11, 'nameservers:': 3, 'ns_records:': 3,
                'targets:': 3
            }
            err_msg = (f'{attribute} was supposed to appear '
                       f'{file_attributes_to_num_of_appearances[attribute]} '
                       'times on the designate-manage output, but '
                       f'it appeared {num_of_occurrences} times.')
            self.assertEqual(
                file_attributes_to_num_of_appearances[attribute],
                num_of_occurrences, err_msg)

    @decorators.idempotent_id('ed42f367-e5ba-40d7-a08d-366ad787d223')
    def test_pool_update_multiple_pools_dry_run(self):
        self._update_pools_file(pools_file_path='multiple-pools.yaml')

        pools_yaml_path = _get_pools_path(name='other-pools.yaml')
        self._run_designate_manage_pool_command(
            'update',
            '--file',
            pools_yaml_path,
            '--dry-run'
        )

        pool_config = self._run_designate_manage_pool_command(
            'show_config', '--all').split('\n\n')[0]

        pool_config_list = pool_config.split('\n')

        for attribute in self.file_attributes_to_num_of_appearances:
            num_of_occurrences = sum(attribute in s for s in pool_config_list)
            file_attributes_to_num_of_appearances = {
                'also_notifies:': 2, 'attributes:': 2, 'description:': 4,
                'id:': 2, 'name:': 7, 'nameservers:': 2, 'ns_records:': 2,
                'targets:': 2
            }
            err_msg = (f'{attribute} was supposed to appear '
                       f'{file_attributes_to_num_of_appearances[attribute]} '
                       'times on the designate-manage output, but '
                       f'it appeared {num_of_occurrences} times.')
            self.assertEqual(
                file_attributes_to_num_of_appearances[attribute],
                num_of_occurrences, err_msg)

    @decorators.idempotent_id('ed42f367-e5ba-40d7-a08d-366ad787d224')
    def test_pool_generate_file(self):
        temp_pools_yaml_conf_path = '/tmp/pools_tempest.yaml'
        if os.path.exists(temp_pools_yaml_conf_path):
            LOG.debug(f'Temporary pools.yaml file {temp_pools_yaml_conf_path} '
                      'exists.\nRemoving this file so we could continue '
                      'testing')
            os.remove(temp_pools_yaml_conf_path)
        self._run_designate_manage_pool_command('generate_file',
                                                '--file',
                                                temp_pools_yaml_conf_path)
        self.assertTrue(os.path.exists(path=temp_pools_yaml_conf_path))
        # (At least) the default pool config should be written to the file
        pools_conf = self._load_config(filename=temp_pools_yaml_conf_path)
        if len(pools_conf) > 1:
            pool_idx = random.randint(0, len(pools_conf) - 1)
        else:
            pool_idx = 0
        pool = yaml.safe_dump(pools_conf[pool_idx]).split('\n')

        for attribute in self.file_attributes_to_num_of_appearances:
            self.assertTrue(
                any(attribute in s for s in pool),
                f'{attribute} not in {pool}'
            )
        if os.path.exists(temp_pools_yaml_conf_path):
            os.remove(temp_pools_yaml_conf_path)


@testtools.skipUnless(CONF.dns_feature_enabled.test_multipool_with_delete_opt,
                      'Multipools feature is being tested with --delete '
                      'option. It might delete pools that were created in '
                      'other tests.')
class DesignateMultiPoolTest(DesignateManagePoolTest):

    _pool1_id = None
    _pool2_id = None

    @property
    def pool1_id(self):
        if not self._pool1_id:
            self._set_pool_ids()
        return self._pool1_id

    @property
    def pool2_id(self):
        if not self._pool2_id:
            self._set_pool_ids()
        return self._pool2_id

    def _set_pool_ids(self):
        pool_config = self._run_designate_manage_pool_command(
            'show_config', '--all').split('\n\n')[0]
        pool_config_list = pool_config.split('\n')
        pool_ids = list()
        for item in pool_config_list:
            if 'id:' in item:
                id = item.split()[1]
                pool_ids.append(id)
        if len(pool_ids) < 2:
            self.skipException('Multipool tests require at least 2 pools.')
        self._pool1_id = pool_ids[0]
        self._pool2_id = pool_ids[1]

    @classmethod
    def setup_clients(cls):
        super(DesignateMultiPoolTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.rec_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.admin_zones_client = cls.os_admin.dns_v2.ZonesClient()

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        super()._update_pools_file(cls.MULTIPOOLS_FILE_PATH)
        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="ZonesTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    def tearDown(self):
        super().tearDown()
        super()._update_pools_file(self.MULTIPOOLS_FILE_PATH)

    @classmethod
    def resource_cleanup(cls):
        super().resource_cleanup()
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])

    @decorators.attr(type='smoke')
    @decorators.attr(type='slow')
    @decorators.idempotent_id('c0648f53-4114-45bd-8792-462a82f69d32')
    def test_create_zones(self):
        LOG.info('Create 2 zones, one per pool')

        zone1_name = dns_data_utils.rand_zone_name(
            name="create_zones_multipool-",
            suffix=self.tld_name)

        zone1 = self.admin_zones_client.create_zone(
            name=f"{zone1_name}",
            attributes={'pool_id': f'{self.pool1_id}'})[1]
        self.addCleanup(self.wait_zone_delete,
                        self.admin_zones_client, zone1['id'],
                        ignore_errors=lib_exc.NotFound)

        zone2_name = dns_data_utils.rand_zone_name(
            name="create_zones_multipool-",
            suffix=self.tld_name)

        zone2 = self.admin_zones_client.create_zone(
            name=f"{zone2_name}",
            attributes={'pool_id': f'{self.pool2_id}'})[1]
        self.addCleanup(self.wait_zone_delete,
                        self.admin_zones_client,
                        zone2['id'],
                        ignore_errors=lib_exc.NotFound)

        self.assertNotEqual(zone1['pool_id'], zone2['pool_id'])

        # wait for both of them to be active
        for zone in [zone1, zone2]:
            waiters.wait_for_zone_status(
                self.admin_zones_client, zone['id'], const.ACTIVE)

            # Create a type A recordset for each zone
            recordset_data = dns_data_utils.rand_recordset_data(
                record_type='A', zone_name=zone['name'])
            body = self.rec_client.create_recordset(zone['id'],
            recordset_data)[1]
            self.addCleanup(
                self.wait_recordset_delete, self.rec_client,
                zone['id'], body['id'])
            self.assertEqual(const.PENDING, body['status'],
                            'Failed, expected status is PENDING')
            waiters.wait_for_recordset_status(
                self.rec_client, zone['id'],
                body['id'], const.ACTIVE)

            for ns in config.CONF.dns.nameservers:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                a_record = str(ns_obj.query(zone['name'], rdatatype='A'))
                self.assertNotEmpty(a_record)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from designate_tempest_plugin import data_utils as utils
from designate_tempest_plugin.services.dns.v2.json import base


class TsigkeyClient(base.DnsClientV2Base):
    """API V2 Tempest REST client for Designate Tsigkey API"""

    @base.handle_errors
    def create_tsigkey(self, resource_id, name=None, algorithm=None,
                       secret=None, scope=None, params=None):
        """Create a tsigkey with the specified parameters.
        :param resource_id: Pool id or Zone id.
        :param name: name of the tsigkey.
        :param algorithm: TSIG algorithm e.g hmac-md5, hmac-sha256 etc.
        :param secret: represents TSIG secret. If provided value is empty
                       it will use empty string (Needed for negative testing)
        :param scope: represents TSIG scope. Default is ZONE
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the created tsigkey.
        """
        tsig = {
                 "name": name or data_utils.rand_name('test-tsig'),
                 "algorithm": algorithm or utils.rand_tsig_algorithm(),
                 "secret": secret or data_utils.rand_name("secret"),
                 "scope": scope or 'ZONE',
                 "resource_id": resource_id}

        if secret == '':
            tsig['secret'] = ''

        resp, body = self._create_request('tsigkeys', data=tsig,
                                          params=params)

        self.expected_success(201, resp.status)

        return resp, body

    @base.handle_errors
    def list_tsigkeys(self, params=None, headers=None):
        """Gets a list of tsigkeys.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized tsigkeys as a list.
        """
        return self._list_request('tsigkeys', params=params, headers=headers)

    @base.handle_errors
    def show_tsigkey(self, uuid=None, params=None, headers=None):
        """Gets a specific tsigkey.
        :param uuid: Unique identifier of the tsigkey in UUID format.
                     Default value is None (it's possible to use "marker" in
                     URL query instead of UUID)
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized tsigkey as a dictionary.
        """
        return self._show_request(
            'tsigkeys', uuid, params=params, headers=headers)

    @base.handle_errors
    def update_tsigkey(self, uuid, name=None, algorithm=None,
                       secret=None, scope=None, params=None):
        """Update the tsigkey with the specified parameters.
        :param uuid: The unique identifier of the tsigkey..
        :param name: name of the tsigkey.
        :param algorithm: TSIG algorithm e.g hmac-md5, hmac-sha256 etc.
        :param secret: represents TSIG secret.
        :param scope: represents TSIG scope.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the updated tsigkey.
        """
        tsig = {
                 "name": name or data_utils.rand_name('test-tsig'),
                 "algorithm": algorithm or utils.rand_tsig_algorithm(),
                 "secret": secret or data_utils.rand_name("secret"),
                 "scope": scope or utils.rand_tsig_scope()}

        resp, body = self._update_request('tsigkeys', uuid, tsig,
                                          params=params)

        self.expected_success(200, resp.status)

        return resp, body

    @base.handle_errors
    def delete_tsigkey(self, uuid, params=None):
        """Deletes a tsigkey having the specified UUID.
        :param uuid: The unique identifier of the tsigkey.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request('tsigkeys', uuid, params=params)

        self.expected_success(204, resp.status)

        return resp, body



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class TldZoneTest(base.BaseDnsV2Test):
    credentials = ["admin", "primary"]
    tld_suffix = '.'.join(["TldZoneTest", CONF.dns.tld_suffix])

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TldZoneTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TldZoneTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.primary_tld_client = cls.os_primary.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(TldZoneTest, cls).resource_setup()
        cls.class_tld = cls.admin_tld_client.create_tld(
            tld_name=cls.tld_suffix)

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(TldZoneTest, cls).resource_cleanup()

    @decorators.idempotent_id('68b3e7cc-bf0e-11ec-b803-201e8823901f')
    def test_create_zone_using_existing_tld(self):
        LOG.info('Creates a zone using existing TLD:"{}"'.format(
            self.tld_suffix))
        zone_name = dns_data_utils.rand_zone_name(
            name='existing_tld_zone', prefix='rand',
            suffix='.{}.'.format(self.tld_suffix))
        zone = self.zones_client.create_zone(
            name=zone_name, wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, zone['id'])

    @decorators.idempotent_id('06deced8-d4de-11eb-b8ee-74e5f9e2a801')
    def test_create_zone_using_not_existing_tld(self):
        LOG.info('Try to create a Zone using not existing TLD:"{}"'.format(
            self.tld_suffix[::-1]))
        zone_name = dns_data_utils.rand_zone_name(
            name='not_existing_tld_zone', prefix='rand',
            suffix='.{}.'.format(self.tld_suffix)[::-1])
        self.assertRaises(
            lib_exc.BadRequest, self.zones_client.create_zone,
            name=zone_name)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg

service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

ServiceAvailableGroup = [
    cfg.BoolOpt("designate",
                default=True,
                help="Whether or not designate is expected to be available."),
]

dns_group = cfg.OptGroup(name='dns',
                         title='DNS service options')

DnsGroup = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the DNS service"),
    cfg.StrOpt('catalog_type',
               default='dns',
               help="Catalog type of the DNS service"),
    cfg.IntOpt('build_interval',
               default=1,
               help="Time in seconds between build status checks."),
    cfg.IntOpt('build_timeout',
               default=360,
               help="Timeout in seconds to wait for an resource to build."),
    cfg.IntOpt('min_ttl',
               default=0,
               help="The minimum value to respect when generating ttl"),
    cfg.ListOpt('nameservers',
                default=[],
                help="The nameservers to check for change going live"),
    cfg.IntOpt('query_timeout',
               default=3,
               help="The timeout on a single dns query to a nameserver"),
    cfg.StrOpt('zone_id',
               help="The target zone to test the dns recordsets "
                    "If it is not specified, a new zone will be created "),
    cfg.StrOpt('tld_suffix',
               default='test',
               help="TLD suffix that used in all tests (if not overridden).")
]

dns_feature_group = cfg.OptGroup(name='dns_feature_enabled',
                                 title='Enabled Designate Features')

DnsFeatureGroup = [
    cfg.BoolOpt('api_v2',
                default=True,
                help="Is the v2 dns API enabled."),
    cfg.BoolOpt('api_admin',
                default=False,
                help="Is the admin dns API enabled."),
    cfg.BoolOpt('api_v2_root_recordsets',
                default=False,
                help="Is the v2 root recordsets API enabled."),
    cfg.BoolOpt('api_v2_quotas',
                default=False,
                help="Is the v2 quota API enabled."),
    cfg.BoolOpt('api_v2_quotas_verify_project',
                default=False,
                help="Is project IDs verified when setting v2 quotas. "
                "Must be set to True starting from Rocky release."),
    cfg.BoolOpt('bug_1932026_fixed',
                default=True,
                deprecated_for_removal=True,
                deprecated_reason='Bug 1932026 was fixed in designate 14.0.0',
                help="Is https://bugs.launchpad.net/designate/+bug/1932026 "
                     "fixed"),
    cfg.StrOpt('designate_manage_path',
               default=None,
               help="The designate-manage command path"),
    # Note: Also see the enforce_scope section (from tempest) for Designate API
    #       scope checking setting.
    cfg.BoolOpt('enforce_new_defaults',
                default=False,
                help='Does the dns service API policies enforce '
                     'the new keystone default roles? This configuration '
                     'value should be same as designate.conf: '
                     '[oslo_policy].enforce_new_defaults option.'),
    cfg.BoolOpt('test_multipool_with_delete_opt',
                default=False,
                help="Is multipool feature being tested with --delete option?"
                     "If it is, it might delete pools that were created in "
                     "other tests."),
]

# Extending this enforce_scope group defined in tempest
enforce_scope_group = cfg.OptGroup(name="enforce_scope",
                                   title="OpenStack Services with "
                                         "enforce scope")
EnforceScopeGroup = [
    cfg.BoolOpt('designate',
                default=False,
                help='Does the dns service API policies enforce '
                     'scope? This configuration value should be same as '
                     'designate.conf: [oslo_policy].enforce_scope option.'),
]



# Copyright 2022 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

from oslo_utils import versionutils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.services.dns.query.query_client import (
    SingleQueryClient)

CONF = config.CONF


# This test suite is intended to test RFC 2317 classless in-addr.arpa
# delegation scenarios.
class ClasslessPTRTest(base.BaseDnsV2Test):

    credentials = ['primary', 'admin', 'alt']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these tests.
        cls.set_network_resources()
        super(ClasslessPTRTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ClasslessPTRTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.zone_client = cls.os_primary.dns_v2.ZonesClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()
        cls.alt_rec_client = cls.os_alt.dns_v2.RecordsetClient()
        cls.share_zone_client = cls.os_primary.dns_v2.SharedZonesClient()

    @classmethod
    def resource_setup(cls):
        super(ClasslessPTRTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        cls.tld_name = '0.192.in-addr-arpa'
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=cls.tld_name)

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(ClasslessPTRTest, cls).resource_cleanup()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('f2d9596c-ce87-4dfd-9bb4-ad2430fd3fe6')
    def test_classless_ptr_delegation(self):
        # Create full subnet zone
        zone_name = f'2.{self.tld_name}.'
        zone = self.zone_client.create_zone(name=zone_name,
                                            wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zone_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        # Create the delegated zone
        delegated_zone_name = f'1-3.2.{self.tld_name}.'
        delegated_zone = self.zone_client.create_zone(
            name=delegated_zone_name, wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zone_client,
                        delegated_zone['id'], ignore_errors=lib_exc.NotFound)

        # Create the PTR record in the delegated zone
        ptr_recordset_data = {
            'name': f'1.1-3.2.{self.tld_name}.',
            'type': 'PTR',
            'records': ['www.example.org.']
        }
        ptr_recordset = self.recordset_client.create_recordset(
            delegated_zone['id'], ptr_recordset_data, wait_until='ACTIVE')[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.recordset_client.delete_recordset,
                        delegated_zone['id'], ptr_recordset['id'])

        # Create the CNAME record
        cname_recordset_data = {
            'name': f'1.2.{self.tld_name}.',
            'type': 'CNAME',
            'records': [f'1.1-3.2.{self.tld_name}.']
        }
        cname_recordset = self.recordset_client.create_recordset(
            zone['id'], cname_recordset_data, wait_until='ACTIVE')[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.recordset_client.delete_recordset,
                        zone['id'], cname_recordset['id'])

        # Check for a CNAME record
        if config.CONF.dns.nameservers:
            ns = config.CONF.dns.nameservers[0]
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_record = ns_obj.query(
                    cname_recordset['name'],
                    rdatatype=cname_recordset_data['type'])
                if cname_recordset_data['records'][0] in str(ns_record):
                    break
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, CNAME record was not detected on '
                        'Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(ns, config.CONF.dns.build_timeout))

        # Check for a PTR record
        if config.CONF.dns.nameservers:
            ns = config.CONF.dns.nameservers[0]
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_record = ns_obj.query(
                    ptr_recordset['name'],
                    rdatatype=ptr_recordset_data['type'])
                if ptr_recordset_data['records'][0] in str(ns_record):
                    break
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, PTR record was not detected on '
                        'Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(ns, config.CONF.dns.build_timeout))

    @decorators.attr(type='slow')
    @decorators.idempotent_id('0110e7b1-9582-410e-b3d5-bd38a1265222')
    def test_classless_ptr_delegation_shared_zone(self):

        if not versionutils.is_compatible('2.1', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'Zone share tests require Designate API version 2.1 or newer. '
                'Skipping test_classless_ptr_delegation_shared_zone test.')

        # Create full subnet zone
        zone_name = f'2.{self.tld_name}.'
        zone = self.zone_client.create_zone(name=zone_name,
                                            wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zone_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        # Create the delegated zone
        delegated_zone_name = f'1-3.2.{self.tld_name}.'
        delegated_zone = self.zone_client.create_zone(
            name=delegated_zone_name, wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zone_client,
                        delegated_zone['id'], ignore_errors=lib_exc.NotFound)

        # Create the CNAME record
        cname_recordset_data = {
            'name': f'1.2.{self.tld_name}.',
            'type': 'CNAME',
            'records': [f'1.1-3.2.{self.tld_name}.']
        }
        cname_recordset = self.recordset_client.create_recordset(
            zone['id'], cname_recordset_data, wait_until='ACTIVE')[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.recordset_client.delete_recordset,
                        zone['id'], cname_recordset['id'])

        # Share the zone with the alt credential
        shared_zone = self.share_zone_client.create_zone_share(
            delegated_zone['id'], self.alt_rec_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        delegated_zone['id'], shared_zone['id'])

        # Create the PTR record in the delegated zone as the alt project
        ptr_recordset_data = {
            'name': f'1.1-3.2.{self.tld_name}.',
            'type': 'PTR',
            'records': ['www.example.org.']
        }
        ptr_recordset = self.alt_rec_client.create_recordset(
            delegated_zone['id'], ptr_recordset_data, wait_until='ACTIVE')[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.alt_rec_client.delete_recordset,
                        delegated_zone['id'], ptr_recordset['id'])

        # Check for a CNAME record
        if config.CONF.dns.nameservers:
            ns = config.CONF.dns.nameservers[0]
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_record = ns_obj.query(
                    cname_recordset['name'],
                    rdatatype=cname_recordset_data['type'])
                if cname_recordset_data['records'][0] in str(ns_record):
                    break
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, CNAME record was not detected on '
                        'Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(ns, config.CONF.dns.build_timeout))

        # Check for a PTR record
        if config.CONF.dns.nameservers:
            ns = config.CONF.dns.nameservers[0]
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_record = ns_obj.query(
                    ptr_recordset['name'],
                    rdatatype=ptr_recordset_data['type'])
                if ptr_recordset_data['records'][0] in str(ns_record):
                    break
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, PTR record was not detected on '
                        'Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(ns, config.CONF.dns.build_timeout))



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.services.dns.v2.json import base


class QuotasClient(base.DnsClientV2Base):

    @base.handle_errors
    def update_quotas(self, project_id, zones=None, zone_records=None,
                      zone_recordsets=None, recordset_records=None,
                      api_export_size=None, params=None,
                      headers=None):
        """Update the quotas for the project id

        :param project_id: Apply the quotas to this project id
        :param zones: The limit on zones per tenant
            Default: Random Value
        :param zone_records: The limit on records per zone
            Default: Random Value
        :param zone_recordsets: The limit recordsets per zone
            Default: Random Value
        :param recordset_records: The limit on records per recordset
            Default: Random Value
        :param api_export_size: The limit on size of on exported zone
            Default: Random Value
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the created quota.
        """
        quotas = dns_data_utils.rand_quotas(
            zones=zones,
            zone_records=zone_records,
            zone_recordsets=zone_recordsets,
            recordset_records=recordset_records,
            api_export_size=api_export_size,
        )

        resp, body = self._update_request('quotas', project_id, quotas,
                                          params=params, headers=headers,
                                          extra_headers=True)

        self.expected_success(200, resp.status)

        return resp, body

    @base.handle_errors
    def show_quotas(self, project_id=None, params=None, headers=None):
        """Gets a specific quota.

        :param project_id: if provided - show the quotas of this project id.
                           https://docs.openstack.org/api-ref/dns/?expanded=
                           get-the-name-servers-for-a-zone-detail#view-quotas
                           If not - show the quotas for a current
                           project.
                           https://docs.openstack.org/api-ref/dns/?expanded=ge
                           t-the-name-servers-for-a-zone-detail#view-current-p
                           roject-s-quotas

        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized quota as a dictionary.
        """
        if project_id is None:
            return self._show_request(
                'quotas', uuid=None, params=params, headers=headers)
        else:
            return self._show_request(
                'quotas', project_id, params=params, headers=headers)

    @base.handle_errors
    def delete_quotas(self, project_id, params=None, headers=None):
        """Resets the quotas for the specified project id

        :param project_id: Reset the quotas of this project id
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request(
            'quotas', project_id,
            params=params, headers=headers,
            extra_headers=True)

        self.expected_success(204, resp.status)

        return resp, body

    @base.handle_errors
    def set_quotas(self, project_id, quotas, params=None, headers=None):
        """Set a specific quota to given project ID

        :param project_id: Set the quotas of this project id.
        :param quotas: Dictionary of quotas to set (POST payload)
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized quota as a dictionary.
        """

        resp, body = self._update_request(
            "quotas", project_id,
            data=quotas, params=params, headers=headers,
            extra_headers=True)
        self.expected_success(200, resp.status)
        return resp, body



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseTransferAcceptTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'key', 'links',
                    'zone_name']

    @classmethod
    def setup_clients(cls):
        super(BaseTransferAcceptTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseTransferAcceptTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseTransferAcceptTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseTransferAcceptTest, cls).resource_cleanup()


class TransferAcceptTest(BaseTransferAcceptTest):
    credentials = ["primary", "alt", "admin",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TransferAcceptTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TransferAcceptTest, cls).setup_clients()

        # Primary clients
        cls.prm_request_client = cls.os_primary.dns_v2.TransferRequestClient()
        cls.prm_accept_client = cls.os_primary.dns_v2.TransferAcceptClient()

        # Alt clients
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.alt_request_client = cls.os_alt.dns_v2.TransferRequestClient()
        cls.alt_accept_client = cls.os_alt.dns_v2.TransferAcceptClient()

        # Admin clients
        cls.admin_zone_client = cls.os_admin.dns_v2.ZonesClient()
        cls.admin_request_client = (cls.os_admin.dns_v2.
                                    TransferRequestClient())
        cls.admin_accept_client = (cls.os_admin.dns_v2.
                                   TransferAcceptClient())

    @decorators.idempotent_id('1c6baf97-a83e-4d2e-a5d8-9d37fb7808f3')
    def test_create_transfer_accept(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_accept", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]
        self.addCleanup(
            self.wait_zone_delete, self.admin_zone_client, zone['id'],
            headers=self.all_projects_header,
            ignore_errors=lib_exc.NotFound)

        LOG.info('Create a zone transfer_request')
        _, transfer_request = self.prm_request_client.create_transfer_request(
            zone['id'])
        self.addCleanup(
            self.transfer_request_delete,
            self.prm_request_client,
            transfer_request['id']
        )

        data = {
                 "key": transfer_request['key'],
                 "zone_transfer_request_id": transfer_request['id']
        }

        # Test RBAC
        # Note: Everyone can call this API and succeed if they know the
        #       transfer key.
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')
            expected_allowed.append('os_project_reader')

        self.check_CUD_RBAC_enforcement(
            'TransferAcceptClient', 'create_transfer_accept',
            expected_allowed, True, data)

        LOG.info('Create a zone transfer_accept')
        _, transfer_accept = self.prm_accept_client.create_transfer_accept(
            data)

        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual('COMPLETE', transfer_accept['status'])

    @decorators.idempotent_id('37c6afbb-3ea3-4fd8-94ea-a426244f019a')
    def test_show_transfer_accept(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(name="show_transfer_accept",
                                              suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]
        self.addCleanup(
            self.wait_zone_delete, self.admin_zone_client, zone['id'],
            headers=self.all_projects_header,
            ignore_errors=lib_exc.NotFound)

        LOG.info('Create a zone transfer_request')
        _, transfer_request = self.prm_request_client.create_transfer_request(
                                  zone['id'])
        self.addCleanup(
            self.transfer_request_delete,
            self.prm_request_client,
            transfer_request['id']
        )

        data = {
            "key": transfer_request['key'],
            "zone_transfer_request_id": transfer_request['id']
        }

        LOG.info('Create a zone transfer_accept')
        _, transfer_accept = self.prm_accept_client.create_transfer_accept(
            data)

        LOG.info('Fetch the transfer_accept')
        _, body = self.prm_accept_client.show_transfer_accept(
            transfer_accept['id'])

        LOG.info('Ensure the fetched response matches the '
                 'created transfer_accept')
        self.assertExpected(transfer_accept, body, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'TransferAcceptClient', 'show_transfer_accept', expected_allowed,
            True, transfer_accept['id'])

        # Test RBAC with x-auth-all-projects
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'TransferAcceptClient', 'show_transfer_accept', expected_allowed,
            True, transfer_accept['id'], headers=self.all_projects_header)

    @decorators.idempotent_id('89b516f0-8c9f-11eb-a322-74e5f9e2a801')
    def test_ownership_transferred_zone(self):

        LOG.info('Create a Primary zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="ownership_transferred_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]
        self.addCleanup(
            self.wait_zone_delete, self.admin_zone_client, zone['id'],
            headers=self.all_projects_header,
            ignore_errors=lib_exc.NotFound)

        LOG.info('Create a Primary zone transfer_request')
        transfer_request = self.prm_request_client.create_transfer_request(
            zone['id'])[1]
        self.addCleanup(
            self.transfer_request_delete,
            self.prm_request_client,
            transfer_request['id']
        )

        data = {
            "key": transfer_request['key'],
            "zone_transfer_request_id": transfer_request['id']
        }
        LOG.info('Create an Alt zone transfer_accept')
        transfer_accept = self.alt_accept_client.create_transfer_accept(
            data)[1]

        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual('COMPLETE', transfer_accept['status'])

        # Make sure that the "project_id" of transferred zone has been changed
        alt_transferred_zone = self.alt_zone_client.show_zone(zone['id'])[1]

        self.assertNotEqual(
            zone['project_id'], alt_transferred_zone['project_id'],
            'Failed, shown "project_id" for a transferred zone:{} should be '
            'different than the original "project_id" used in '
            'creation {}:'.format(
                alt_transferred_zone['project_id'], zone['project_id']))

    @decorators.idempotent_id('0fcd314c-8cae-11eb-a322-74e5f9e2a801')
    def test_list_transfer_accepts(self):
        """Test list API including filtering result option"""

        number_of_zones_to_transfer = 3
        transfer_request_ids = []
        for _ in range(number_of_zones_to_transfer):

            LOG.info('Create a Primary zone')
            zone_name = dns_data_utils.rand_zone_name(
                name="list_transfer_accepts", suffix=self.tld_name)
            zone = self.zones_client.create_zone(name=zone_name,
                                                 wait_until='ACTIVE')[1]
            self.addCleanup(
                self.wait_zone_delete, self.admin_zone_client, zone['id'],
                headers=self.all_projects_header,
                ignore_errors=lib_exc.NotFound)

            LOG.info('Create a Primary zone transfer_request')
            transfer_request = self.prm_request_client.create_transfer_request(
                zone['id'])[1]
            self.addCleanup(
                self.transfer_request_delete,
                self.prm_request_client,
                transfer_request['id']
            )

            data = {
                "key": transfer_request['key'],
                "zone_transfer_request_id": transfer_request['id']
            }
            LOG.info('Create an Alt zone transfer_accept')
            transfer_accept = self.alt_accept_client.create_transfer_accept(
                data)[1]

            LOG.info('Ensure we respond with COMPLETE status')
            self.assertEqual('COMPLETE', transfer_accept['status'])
            transfer_request_ids.append(transfer_accept['id'])

        # Test RBAC - Users that are allowed to call list, but should get
        #             zero zones.
        expected_allowed = ['os_admin']

        self.check_list_RBAC_enforcement_count(
            'TransferAcceptClient', 'list_transfer_accept',
            expected_allowed, 0)

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'TransferAcceptClient', 'list_transfer_accept',
            expected_allowed, transfer_request_ids,
            headers=self.all_projects_header)

        # As Admin list all accepted zone transfers, expected:
        # each previously transferred zone is listed.
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        LOG.info('Use Admin client to list all "accepted zone transfers"')
        admin_client_accept_ids = [
            item['id'] for item in
            self.admin_accept_client.list_transfer_accept(
                headers=self.all_projects_header,
                params={'limit': 1000})[1]['transfer_accepts']]
        for tr_id in transfer_request_ids:
            self.assertIn(
                tr_id, admin_client_accept_ids,
                'Failed, expected transfer accept ID:{} is not listed in'
                ' transfer accept IDs:{} '.format(tr_id, transfer_request_ids))

        # As Admin list all accepted zone transfers in COMPLETE status only,
        # expected: each previously transferred zone is listed.
        LOG.info('Use Admin client to list all "accepted zone transfers", '
                 'filter COMPLETE status only accepts.')
        admin_client_accept_ids = [
            item['id'] for item in
            self.admin_accept_client.list_transfer_accept(
                headers=self.all_projects_header,
                params={'status': 'COMPLETE'})[1]['transfer_accepts']]
        for tr_id in transfer_request_ids:
            self.assertIn(
                tr_id, admin_client_accept_ids,
                'Failed, expected transfer accept ID:{} is not listed in'
                ' transfer accept IDs:{} '.format(
                    tr_id, transfer_request_ids))

        # As Admin list all accepted zone transfers in "non existing" status,
        # expected: received list is empty.
        not_existing_status = 'zahlabut'
        LOG.info('Use Admin client to list all "accepted zone transfers", '
                 'filter {} status only accepts.'.format(not_existing_status))
        admin_client_accept_ids = [
            item['id'] for item in
            self.admin_accept_client.list_transfer_accept(
                headers=self.all_projects_header,
                params={'status': not_existing_status})[1]['transfer_accepts']]
        self.assertEmpty(
            admin_client_accept_ids,
            "Failed, filtered list should be empty, but actually it's not, "
            "filtered IDs:{} ".format(admin_client_accept_ids))

    @decorators.idempotent_id('b6ac770e-a1d3-11eb-b534-74e5f9e2a801')
    def test_show_transfer_accept_impersonate_another_project(self):
        LOG.info('Create a zone as primary tenant')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_transfer_accept_impersonate", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]

        # In case when something goes wrong with the test and E2E
        # scenario fails for some reason, we'll use Admin tenant
        # to activate Cleanup for a zone.
        # Note: "ignore_errors=lib_exc.NotFound" is used to prevent a
        # failure in case when E2E scenario was successfully completed.
        # Means that Alt tenant has already been able to run a cleanup
        # for a zone.
        self.addCleanup(
            self.wait_zone_delete, self.admin_zone_client, zone['id'],
            headers=self.all_projects_header,
            ignore_errors=lib_exc.NotFound)

        LOG.info('Create a zone transfer_request as primary tenant')
        transfer_request = self.prm_request_client.create_transfer_request(
                                  zone['id'])[1]
        self.addCleanup(
            self.transfer_request_delete,
            self.prm_request_client,
            transfer_request['id']
        )
        data = {
            "key": transfer_request['key'],
            "zone_transfer_request_id": transfer_request['id']
        }

        LOG.info('Create a zone transfer_accept for Alt tenant, using '
                 'Admin client and "sudo" option')
        transfer_accept = self.admin_accept_client.create_transfer_accept(
            data, headers={
                'x-auth-sudo-project-id': self.os_alt.credentials.project_id,
                'content-type': 'application/json'})[1]

        LOG.info('Fetch the transfer_accept as Alt tenant')
        body = self.alt_accept_client.show_transfer_accept(
            transfer_accept['id'])[1]

        LOG.info('Ensure the fetched response matches the '
                 'created transfer_accept')
        self.assertExpected(transfer_accept, body, self.excluded_keys)

        # E2E accept zone transfer is done, therefore Alt tenant
        # should be able to "cleanup" a transferred zone.
        self.addCleanup(
            self.wait_zone_delete, self.alt_zone_client, zone['id'])

        # Test RBAC with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'TransferAcceptClient', 'show_transfer_accept', expected_allowed,
            True, transfer_accept['id'],
            headers={'x-auth-sudo-project-id':
                     self.os_alt.credentials.project_id})


class TransferAcceptTestNegative(BaseTransferAcceptTest):

    credentials = ["primary", "alt", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TransferAcceptTestNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TransferAcceptTestNegative, cls).setup_clients()
        cls.request_client = cls.os_primary.dns_v2.TransferRequestClient()
        cls.client = cls.os_primary.dns_v2.TransferAcceptClient()

    @decorators.idempotent_id('324a3e80-a1cc-11eb-b534-74e5f9e2a801')
    def test_create_transfer_accept_using_invalid_key(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_accept_invalid_key", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.request_client.create_transfer_request(
                                  zone['id'])[1]
        self.addCleanup(
            self.transfer_request_delete,
            self.request_client,
            transfer_request['id']
        )

        data = {"key": data_utils.rand_password(
                len(transfer_request['key'])),
                "zone_transfer_request_id": transfer_request['id']}

        LOG.info('Create a zone transfer_accept using invalid key')
        self.assertRaises(
            lib_exc.Forbidden, self.client.create_transfer_accept,
            transfer_accept_data=data)

    @decorators.idempotent_id('23afb948-a1ce-11eb-b534-74e5f9e2a801')
    def test_create_transfer_accept_using_deleted_transfer_request_id(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_accept_deleted_id", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.request_client.create_transfer_request(
                                  zone['id'])[1]
        self.addCleanup(
            self.transfer_request_delete,
            self.request_client,
            transfer_request['id']
        )

        data = {
                 "key": transfer_request['key'],
                 "zone_transfer_request_id": transfer_request['id']
        }

        LOG.info('Delete transfer request')
        self.request_client.delete_transfer_request(transfer_request['id'])

        LOG.info('Ensure 404 when accepting non existing request ID')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.create_transfer_accept(data))



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import requests

from oslo_log import log as logging
from tempest.lib import decorators

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.services.dns.v2.json import base as service_base

from tempest import config

CONF = config.CONF
LOG = logging.getLogger(__name__)


class DesignateApiVersion(base.BaseDnsV2Test, service_base.DnsClientV2Base):
    credentials = ['admin', 'primary']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(DesignateApiVersion, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(DesignateApiVersion, cls).setup_clients()

        cls.admin_client = cls.os_admin.dns_v2.ApiVersionClient()
        cls.primary_client = cls.os_primary.dns_v2.ApiVersionClient()

    @decorators.idempotent_id('aa84986e-f2ad-11eb-b58d-74e5f9e2a801')
    def test_list_enabled_api_versions(self):
        for user in ['admin', 'primary', 'not_auth_user']:
            if user == 'admin':
                ver_doc = self.admin_client.list_enabled_api_versions()[1]
                # The version document was updated to match OpenStack
                # version discovery standards in Zed. Accomodate the legacy
                # format for backward compatibility.
                try:
                    versions = ver_doc['versions']['values']
                except TypeError:
                    versions = ver_doc['versions']
            if user == 'primary':
                ver_doc = self.primary_client.list_enabled_api_versions()[1]
                # The version document was updated to match OpenStack
                # version discovery standards in Zed. Accomodate the legacy
                # format for backward compatibility.
                try:
                    versions = ver_doc['versions']['values']
                except TypeError:
                    versions = ver_doc['versions']
            if user == 'not_auth_user':
                response = requests.get(self.primary_client.base_url,
                                        verify=False)
                headers = {
                    k.lower(): v.lower() for k, v in response.headers.items()}
                # The version document was updated to match OpenStack
                # version discovery standards in Zed. Accomodate the legacy
                # format for backward compatibility.
                try:
                    versions = self.deserialize(
                        headers, str(response.text))['versions']['values']
                except TypeError:
                    versions = self.deserialize(
                        headers, str(response.text))['versions']

            LOG.info('Received enabled API versions for {} '
                     'user are:{}'.format(user, versions))

            enabled_ids = {item['id'] for item in versions}
            LOG.info('Enabled versions IDs are:{}'.format(enabled_ids))

            # Expect at least one of these to be present in the API
            base_versions = {'v1', 'v2', 'v2.0'}
            self.assertFalse(
                enabled_ids.isdisjoint(base_versions),
                'Failed, at least one base API version: {} was not found in '
                'the API version list: {}'.format(base_versions, enabled_ids))
